#! /bin/sh
#
# @(#)$Id$
#
# (C) Copyright 2009 Nikhef, National Institute for Subatomic Physics
#                    David Groep, Grid PDP group
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may 
# not use this file except in compliance with the License. You may obtain 
# a copy of the License at 
#       http://www.apache.org/licenses/LICENSE-2.0 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License.
#
# Purpose:
#   Set the VO_xxx_CACHE_DIR variable when appropriate, based on a list of
#   possible base directories for such a cache. The script will accept any
#   incoming VONAME definition (such as set by the globus Job Manager patch)
#   but failing that will try to determine the VO based on the VOMS proxy.
#
# Configuration: will read /etc/sysconfig/vocachedir
#   Set CACHEBASES to a list of directory paths to which the voname (in
#   lower case) will be appended to name the actual cache directory. Its
#   permissions MUST be at least 01755 (i.e. the directory MUST be sticky)
#   and it MUST be owned by root
#
# Note: the script will NOT do cleanup of this directory, use tmpwatch(8) 
# for that.

cacheverbose=${cacheverbose:-0}
vomsproxyinfo=/opt/glite/bin/voms-proxy-info
CACHEBASES="/var/cache/grid /tmp/gridcache"

if test -f /etc/sysconfig/vocachedir
then
  . /etc/sysconfig/vocachedir
fi

voclog() {
  if test "$cacheverbose" -ne 0 
  then
    echo "$@" >&2
  fi
}

if test -z "$VONAME" -a -n "$X509_USER_PROXY"
then
  if test -x $vomsproxyinfo 
  then
    voname=$($vomsproxyinfo -file "$X509_USER_PROXY" -vo 2>/dev/null)
    case "$voname" in
    [a-zA-Z][-a-zA-Z.]* )	
      VONAME="$voname"
      export VONAME
      if test -n "$ISCSHELL"
      then
        echo "setenv VONAME \"$voname\""
      fi
      ;;
    * )
      unset voname
      ;;
    esac
  fi
fi

# utility function to set it (allows local vars)
setvocachedir() {
  local voucalias volcalias dirperm dir

  voucalias=$(echo "$VONAME" | tr a-z A-Z | sed -e 's/[^a-zA-Z0-9]/_/g')
  volcalias=$(echo "$VONAME" | tr A-Z a-z | sed -e 's/[^a-zA-Z0-9]/_/g')
  for dir in $CACHEBASES
  do
    voclog "Testing availability of cache base $dir"
    if test -d "$dir/$volcalias"
    then
      dirperm=$(stat -c %a "$dir/$volcalias")
      dirowner=$(stat -c %u "$dir/$volcalias")
      voclog "Testing $dir/$volcalias (perm=$dirperm,uid=$dirowner)"
      if test "$dirperm" -lt 1755
      then
        voclog Cache $dir/$volcalias exists but permissions $dirperm too lax
        voclog Declining to set VO_${voucalias}_CACHE_DIR
        return 1
      fi
      if test "$dirowner" -ne 0
      then
        voclog Cache $dir/$volcalias exists but owner is not root but $dirowner
        voclog Declining to set VO_${voucalias}_CACHE_DIR
        return 1
      fi
      eval VO_${voucalias}_CACHE_DIR="$dir/$volcalias"
      if test -n "$ISCSHELL"
      then
        echo "setenv VO_${voucalias}_CACHE_DIR \"$dir/$volcalias\""
      fi
      break
    fi
  done
  return 0;
}

# we have figured out the VO name if it is to be found at all
# only continue if the VO is set, as we do not offer a generic cache
if test -n "$VONAME"
then
  setvocachedir
fi

unset setvocachedir
unset voclog
unset voname
